﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Order.Read.Data;
using Order.Read.Domain;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

namespace Order.Read.Tests
{
  [TestClass]
  public class UnitTest1
  {
    [TestMethod]
    public void CanQueryDataFromReadModel() {
      //arrange --.
      var scContext = new OrderReadContext { Orders = MockedDbSet<SalesOrder>(null) };
      var readmodel = new ReadModel();

      //act and assert
      Assert.IsInstanceOfType(readmodel.Orders.ToList(), typeof(List<SalesOrder>));
    }

    private static DbSet<T> MockedDbSet<T>(ICollection<T> inMemoryEntities) where T : class {
      if (inMemoryEntities == null) {
        inMemoryEntities = new List<T>();
      }
      var queryableData = inMemoryEntities.AsQueryable();
      var mockedDbSet = new Mock<DbSet<T>>();

      mockedDbSet.Setup(m => m.Add(It.IsAny<T>()))
        .Callback<T>(inMemoryEntities.Add);
      mockedDbSet.As<IQueryable<T>>().Setup(m => m.Provider).Returns(queryableData.Provider);
      mockedDbSet.As<IQueryable<T>>().Setup(m => m.Expression).Returns(queryableData.Expression);
      mockedDbSet.As<IQueryable<T>>().Setup(m => m.ElementType).Returns(queryableData.ElementType);
      mockedDbSet.As<IQueryable<T>>().Setup(m => m.GetEnumerator()).Returns(queryableData.GetEnumerator());
      mockedDbSet.Setup(x => x.AsNoTracking()).Returns(mockedDbSet.Object);

      return mockedDbSet.Object;
    }
  }
}
